@@ -1,4 +1,4 @@
-;;; ecb-analyse.el --- ECB analysis display interactor
+;;; ecb-analyse.el --- ECB analysis display window
 
 ;;; Copyright (C) 2004 - 2005 Klaus Berndl
 
@@ -20,7 +20,7 @@
 ;; GNU Emacs; see the file COPYING.  If not, write to the Free Software
 ;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-;; $Id: ecb-analyse.el,v 1.13 2005/06/27 17:03:06 berndl Exp $
+;; $Id: ecb-analyse.el,v 1.25 2009/05/08 14:05:55 berndl Exp $
 
 
 ;;; Commentary:
@@ -73,9 +73,9 @@ key-bindings only for the analyse-buffer of ECB."
   :type 'hook)
 
 (defcustom ecb-analyse-show-node-info '(if-too-long . name)
-  "*When to display which node-info in the history-buffer.
+  "*When to display which node-info in the analyse-buffer.
 Define which node info should displayed after moving the mouse over a node
-\(or after a shift click onto the node) in the history-buffer.
+\(or after a shift click onto the node) in the analyse-buffer.
 
 You can define \"when\" a node-info should be displayed:
 See `ecb-directories-show-node-info' for the possible choices.
@@ -110,8 +110,6 @@ method is the context. In object oriented languages this can be the full
 hierachy, i.e. not only the current method, but the current method, the class
 of this method, the superclass of this class and so on!
 
-Arguments: The arguments of the context if the context is a function/method.
-
 Local Variables: All accessible and bound local variables visible at current
 point.
 
@@ -143,14 +141,16 @@ bucket. So most needed buckets are better visible in the analyse-buffer."
                          (const :tag "Completions" :value "Completions")
                          (string :tag "Other bucketname"))))
 
-(defcustom ecb-analyse-fontified-buckets '("Context")
+(defcustom ecb-analyse-fontified-buckets '("Context" "Function")
   "*Buckets whose elements should be fontified as in the methods-buffer.
-If the name of a category/bucket is contained in this option then all elements
-of this bucket will be displayed as in the methods-buffer - at least if an
-element is a semantic-tag. This means if `ecb-font-lock-tags' is not nil these
-elements will be fontified and also displayed with an appropriate icon if
-possible. The default value does this only for the Context-bucket because for
-most of the other buckets this makes not really much sense.
+If the name of a category/bucket is contained in this option then
+all elements of this bucket will be displayed as in the
+methods-buffer - at least if an element is a semantic-tag. This
+means if `ecb-font-lock-tags' is not nil and the font-lock
+feature is loaded into Emacs these elements will be fontified and
+also displayed with an appropriate icon if possible. The default
+value does this only for the Context-bucket because for most of
+the other buckets this makes not really much sense.
 
 For available buckets see `ecb-analyse-collapsed-buckets'.
 
@@ -201,18 +201,81 @@ See also `ecb-analyse-gen-tag-info-fn'."
                        :value ecb-analyse-show-tag-info-in-temp-buffer)
                 (function :tag "Info display-function")))
 
-;; TODO: Klaus Berndl <klaus.berndl@sdm.de>: first we start simple by creating
-;; an analyse-tree-buffer which can be included in a layout and all should
-;; work. Later we could design a basic-mechanism which:
-;; - creates a "mode"-tree-buffer for stuff like analyse, class browser etc.
-;; - a mechansims which copy the anaylse-tree-buffer (or in future the
-;;   class-browser-tree-buffer etc.) to that "mode"-tree-buffer so you can
-;;   display in that basic tree-buffer different "modes". Probably this would
-;;   be the best: We have 4 basic tree-buffers (directories, sources, methods
-;;   and history and one additional "mode"-tree-buffer which can be added to a
-;;   layout. Then there is a command which display different tree-buffers uin
-;;   the mode-tree-buffer, e.g. the analyse tree-buffer or a
-;;   class-browser-tree-buffer.
+(defcustom ecb-analyse-buffer-sync 'basic
+  "*Synchronize the analyse buffer automatically with current edit buffer.
+
+If 'always then the synchronization takes place always a buffer changes in the
+edit window, if nil then never. If a list of major-modes then only if the
+`major-mode' of the new buffer belongs NOT to this list.
+
+Normally it's not necessary to exclude some major-modes because with
+not-semantic supported major-modes simply nothing happens. But maybe it can be
+helpful for certain situations...
+
+If the special value 'basic is set then ECB uses the setting of the option
+`ecb-basic-buffer-sync'.
+
+IMPORTANT NOTE: Every time the synchronization is done the hook
+`ecb-analyse-buffer-sync-hook' is evaluated."
+    :group 'ecb-analyse
+    :type '(radio :tag "Synchronize ECBs analyse buffer"
+                  (const :tag "use basic value" :value basic)
+                  (const :tag "Always" :value always)
+                  (const :tag "Never" nil)
+                  (repeat :tag "Not with these modes"
+                          (symbol :tag "mode"))))
+
+(defcustom ecb-analyse-buffer-sync-delay 2
+  "*Time Emacs must be idle before the analyse-buffer is synchronized.
+Synchronizing is done with the current source displayed in the edit window. If
+nil then there is no delay, means synchronization takes place immediately.
+
+CAUTION: With analysing a value not too small is strongly recommended because
+it can be very annoying if more or less after each typing the current context
+is analysed. If set to nil then *each* keyboard hit refreshes the
+analyse-buffer which will make ECB quite unusable!
+
+If the special value 'basic is set then ECB uses the setting of the option
+`ecb-basic-buffer-sync-delay'"
+  :group 'ecb-analyse
+  :type '(radio (const :tag "Use basic value" :value basic)
+                (const :tag "No synchronizing delay" :value nil)
+                (number :tag "Idle time before synchronizing" :value 2))
+  :set (function (lambda (symbol value)
+                   (set symbol value)
+                   (if (and (boundp 'ecb-minor-mode)
+                            ecb-minor-mode)
+                       (ecb-activate-ecb-autocontrol-function
+                        value 'ecb-analyse-buffer-sync))))
+  :initialize 'custom-initialize-default)
+
+(defcustom ecb-analyse-buffer-sync-hook nil
+  "Hook run at the end of `ecb-analyse-buffer-sync'.
+See documentation of `ecb-analyse-buffer-sync' for conditions when
+synchronization takes place and so in turn these hooks are evaluated.
+
+Preconditions for such a hook:
+- Current buffer is the buffer of the currently selected
+  edit-window.
+- The analyse-buffer is displayed in a visible window of the
+  ecb-frame \(so no check for visibilty of the analyse-buffer in
+  the ecb-frame is necessary in a hook function)
+
+Postcondition for such a hook:
+Point must stay in the same edit-window as before evaluating the hook.
+
+Important note: If the option `ecb-analyse-buffer-sync' is not
+nil the function `ecb-analyse-buffer-sync' is running either
+every time Emacs is idle or even after every command \(see
+`ecb-analyse-buffer-sync-delay'). So if the anaylse-buffer is
+displayed in a window of the ecb-frame \(see preconditions above)
+these hooks can be really called very often! Therefore each
+function of this hook should/must check in an efficient way at
+beginning if its task have to be really performed and then do
+them only if really necessary! Otherwise performance of Emacs
+could slow down dramatically!"
+    :group 'ecb-analyse
+    :type 'hook)
 
 (defconst ecb-analyse-nodedata-tag-with-pos 0)
 (defconst ecb-analyse-nodedata-tag-without-pos 1)
@@ -228,50 +291,68 @@ See also `ecb-analyse-gen-tag-info-fn'."
 (defconst ecb-analyse-nodetype-function 7)
 (defconst ecb-analyse-nodetype-function-arg 8)
 
-(defun ecb-analyse-buffer-sync ()
+(defecb-autocontrol/sync-function ecb-analyse-buffer-sync
+    ecb-analyse-buffer-name ecb-analyse-buffer-sync t
   "Synchronize the analyse buffer with the current buffer and point.
 This means in fact display the current analysis for current point."
-  (interactive)
-  (ecb-do-if-buffer-visible-in-ecb-frame 'ecb-analyse-buffer-name
-    ;; (current-buffer) is here the current buffer of the edit-area!
-    (let ((analysis nil)
-          (completions nil)
-          (fnargs nil)
-          (cnt nil)
-          )
-      ;; Try and get some sort of analysis
-      (ignore-errors
-        (save-excursion
-          (setq analysis (ecb--semantic-analyze-current-context (point)))
-          (setq cnt (ecb--semantic-find-tag-by-overlay))
-          (when analysis
-            (setq completions (ecb--semantic-analyze-possible-completions analysis))
-            (setq fnargs (ecb--semantic-get-local-arguments (point)))
-            )))
-      (ecb-exec-in-window ecb-analyse-buffer-name
-        ;; we must remove the old nodes
-        (tree-buffer-set-root (tree-node-new-root))
-        (when analysis
-          ;; Now insert information about the context
-          (when cnt
-            (ecb-analyse-add-nodes "Context" "Context"
-                                   cnt ecb-analyse-nodetype-context))
-          (when fnargs
-            (ecb-analyse-add-nodes "Arguments" "Arguments" fnargs
-                                   ecb-analyse-nodetype-arguments))
-          ;; Let different classes draw more nodes.
-          (ecb-analyse-more-nodes analysis)
-          (when completions
-            (ecb-analyse-add-nodes "Completions" "Completions" completions
-                                   ecb-analyse-nodetype-completions)))
-        (tree-buffer-update)))))
+  ;; TODO: Klaus Berndl <klaus.berndl@sdm.de>: make interruptable. Necessary
+  ;; e.g. when typing: "(e" then scanning all elisp stuff beginning with e is
+  ;; really annoying....
+  (let ((analysis nil)
+        (scope nil)
+        (completions nil)
+        (cnt nil)
+        (mode-local-active-mode nil)
+        )
+    ;; Try and get some sort of analysis
+    (condition-case nil
+        (progn
+          (setq mode-local-active-mode major-mode)
+          (save-excursion
+            ;; Get the current scope
+            (setq scope (semantic-calculate-scope (point)))
+            ;; Get the analysis
+            (setq analysis (ecb--semantic-analyze-current-context (point)))
+            (setq cnt (ecb--semantic-find-tag-by-overlay))
+            (when analysis
+              (setq completions (ecb--semantic-analyze-possible-completions analysis)))))
+      (error nil))
+    (ecb-exec-in-window ecb-analyse-buffer-name
+      ;; we must remove the old nodes
+      (tree-buffer-set-root (tree-node-new-root))
+      (when cnt
+        (ecb-analyse-add-nodes "Context" "Context"
+                               cnt ecb-analyse-nodetype-context))
+      ;; TODO: Klaus Berndl <klaus.berndl@sdm.de>: maybe we should adopt this
+      ;; for ecb-analyse..
+      ;;     (when analysis
+      ;;       ;; If this analyzer happens to point at a complete symbol, then
+      ;;       ;; see if we can dig up some documentation for it.
+      ;;       (semantic-ia-sb-show-doc analysis))
+
+      ;; Show local variables
+      (when scope
+        (ecb-analyse-show-scope scope))
+
+      (when analysis
+        ;; Let different classes draw more buttons.
+        (ecb-analyse-more-nodes analysis)
+        (when completions
+          (ecb-analyse-add-nodes "Completions" "Completions" completions
+                                 ecb-analyse-nodetype-completions)))
+      (tree-buffer-update)))
+  (run-hooks 'ecb-analyse-buffer-sync-hook))
+        
+(defun ecb-analyse-show-scope (scope)
+  "Show SCOPE information."
+  (let ((localvars (when scope
+		     (oref scope localvar))))
+    (when localvars
+      (ecb-analyse-add-nodes "Local Variables" "Local Variables" localvars
+                             ecb-analyse-nodetype-localvars))))
 
 (defmethod ecb-analyse-more-nodes ((context semantic-analyze-context))
   "Show a set of ecb-nodes specific to CONTEXT."
-  (let ((localvars (oref context localvariables)))
-    (when localvars
-      (ecb-analyse-add-nodes "Local Variables" "Local Variables" localvars
-                             ecb-analyse-nodetype-localvars)))
   (let ((prefix (oref context prefix)))
     (when prefix
       (ecb-analyse-add-nodes "Prefix" "Prefix" prefix ecb-analyse-nodetype-prefix))))
@@ -313,8 +394,9 @@ of LIST."
   (when list
     (save-excursion
       (set-buffer ecb-analyse-buffer-name)
-      (let* ((bucket-name-formatted (ecb-merge-face-into-text bucket-name
-                                                              ecb-analyse-bucket-node-face))
+      (let* ((bucket-name-formatted
+              (ecb-merge-face-into-text (ecb-format-bucket-name bucket-name)
+                                        ecb-analyse-bucket-node-face))
              (bucket-node (tree-node-new bucket-name-formatted
                                          ecb-analyse-nodetype-bucket
                                          (list 'ecb-bucket-node
@@ -322,32 +404,33 @@ of LIST."
                                          nil
                                          (tree-buffer-get-root))))
         (setf (tree-node->expanded bucket-node)
-              (not (member bucket
-                           ecb-analyse-collapsed-buckets)))
-        (dolist (elem list)
-          (let* ((fontify-tags (member bucket ecb-analyse-fontified-buckets))
-                 (string-1 (typecase elem
-                             (string elem)
-                             (ecb--semantic-tag
-                              (if fontify-tags
-                                  (ecb-displayed-tag-name elem)
-                                (ecb--semantic-format-tag-uml-concise-prototype elem)))
-                             (otherwise "foo")))
-                 (string (concat string-1)))
-            (unless fontify-tags
-              (ecb-merge-face-into-text string ecb-analyse-bucket-element-face))
-            (if (ecb--semantic-tag-p elem)
+              (not (member bucket ecb-analyse-collapsed-buckets)))
+        (ecb-exit-on-input 'ecb-analyse
+          (dolist (elem list)
+            (ecb-throw-on-input 'ecb-analyse-tree-buffer-build)
+            (let* ((fontify-tags (member bucket ecb-analyse-fontified-buckets))
+                   (string-1 (typecase elem
+                               (string elem)
+                               (ecb--semantic-tag
+                                (if fontify-tags
+                                    (ecb-displayed-tag-name elem)
+                                  (ecb--semantic-format-tag-uml-concise-prototype elem)))
+                               (otherwise "foo")))
+                   (string (concat string-1)))
+              (unless fontify-tags
+                (ecb-merge-face-into-text string ecb-analyse-bucket-element-face))
+              (if (ecb--semantic-tag-p elem)
+                  (tree-node-new string nodetype
+                                 (list elem 
+                                       (if (ecb--semantic-tag-with-position-p elem)
+                                           ecb-analyse-nodedata-tag-with-pos
+                                         ecb-analyse-nodedata-tag-without-pos)
+                                       nodetype)
+                                 t bucket-node nil)
                 (tree-node-new string nodetype
-                               (list elem
-                                     (if (ecb--semantic-tag-with-position-p elem)
-                                         ecb-analyse-nodedata-tag-with-pos
-                                       ecb-analyse-nodedata-tag-without-pos)
-                                     nodetype)
-                               t bucket-node nil)
-              (tree-node-new string nodetype
-                             (list elem ecb-analyse-nodedata-no-tag nodetype)
-                             t bucket-node nil))))))))
-
+                               (list elem ecb-analyse-nodedata-no-tag nodetype)
+                               t bucket-node nil)))))))))
+  
 (defun ecb-analyse-compare-node-data (left right)
   "Return not nil when LEFT and RIGHT are identical node-datas."
   (and (equal (nth 2 left) (nth 2 right))
@@ -365,55 +448,58 @@ used as window."
         ;; if we have a positioned tag we jump to it
         (when (and tag (= (nth 1 data) ecb-analyse-nodedata-tag-with-pos))
           ;; We must highlight the tag
-          (tree-buffer-highlight-node-data data)
-          (ecb-jump-to-tag (or (and (ecb--semantic-tag-buffer tag)
-                                    (buffer-file-name (ecb--semantic-tag-buffer tag)))
-                               ;; then we have a tag with no buffer but only
-                               ;; buffer-start- and buffer-end-pos
-                               ecb-path-selected-source)
+          (tree-buffer-highlight-node-by-data/name data)
+          ;; TODO: Klaus Berndl <klaus.berndl@sdm.de>: what about tags without
+          ;; buffer but onlxy with start- and end-pos?!
+          (ecb-display-tag (ecb-source-make (ecb-buffer-file-name
+                                             (ecb-semantic-tag-buffer tag))
+                                            (ecb-semantic-tag-buffer tag))
                            tag
-                           (or window ecb-last-edit-window-with-point)
+                           (or window (ecb-get-edit-window nil))
                            t nil))))))
 
-(tree-buffer-defpopup-command ecb-analyse-complete
-  "Complete at current point of the edit-window the selected completion-tag."
+(tree-buffer-defpopup-command ecb-analyse-complete/insert
+  "Complete/insert at current point the selected completion/localvar."
   ;; We must highlight the tag
   (let* ((data (tree-node->data node))
          (tag (nth 0 data))
          (type (tree-node->type node)))
-    (when (= type ecb-analyse-nodetype-completions)
-      (tree-buffer-highlight-node-data data)
-      (ecb-find-file-and-display ecb-path-selected-source nil)
+    (when (or (= type ecb-analyse-nodetype-completions)
+              (= type ecb-analyse-nodetype-localvars))
+      (tree-buffer-highlight-node-by-data/name data)
+      (ecb-display-source ecb-path-selected-source nil)
       (let* ((a (ecb--semantic-analyze-current-context (point)))
-             (bounds (oref a bounds))
+             (bounds (if a (oref a bounds)))
              (movepoint nil))
-        (save-excursion
-          (if (and (<= (point) (cdr bounds)) (>= (point) (car bounds)))
-              (setq movepoint t))
-          (goto-char (car bounds))
-          (delete-region (car bounds) (cdr bounds))
-          (insert (ecb--semantic-tag-name tag))
-          (if movepoint (setq movepoint (point))))
-        (if movepoint
-            (goto-char movepoint))))))
+        (if (null bounds)
+            (insert (ecb--semantic-tag-name tag))
+          (save-excursion
+            (if (and (<= (point) (cdr bounds)) (>= (point) (car bounds)))
+                (setq movepoint t))
+            (goto-char (car bounds))
+            (delete-region (car bounds) (cdr bounds))
+            (insert (ecb--semantic-tag-name tag))
+            (if movepoint (setq movepoint (point))))
+          (if movepoint
+              (goto-char movepoint)))))))
 
-(defun ecb-analyse-node-clicked (node ecb-button edit-window-nr
-                                      shift-mode meta-mode)
-  "Handle clicking onto NODE in the analyse-buffer. ECB-BUTTON can be 1, 2 or
-3. If 3 then EDIT-WINDOW-NR contains the number of the edit-window the NODE
-should be displayed or whatever should be done with NODE. For 1 and 2 the
-value of EDIT-WINDOW-NR is ignored."
+(defecb-tree-buffer-callback ecb-analyse-node-clicked ecb-analyse-buffer-name select nil
+  "Handles clicking onto any of the nodes in the analyse-buffer of ECB."
   (if shift-mode
       (ecb-mouse-over-analyse-node node nil nil 'force))
   (let* ((data (tree-node->data node))
          (tag (nth 0 data))
          (type (tree-node->type node)))
+    ;; we handle hiding the ecb-windows for ourself
+    (setq no-meta-hiding t)
     (cond
      ((= type ecb-analyse-nodetype-bucket)
       (tree-node-toggle-expanded node)
       (tree-buffer-update node))
      ((= type ecb-analyse-nodetype-completions)
-      (ecb-analyse-complete node))
+      (ecb-analyse-complete/insert node))
+     ((= type ecb-analyse-nodetype-localvars)
+      (ecb-analyse-complete/insert node))
      (t
       (ecb-analyse-jump-to-tag node (ecb-get-edit-window
                                      ;; `ecb-analyse-jump-to-tag' expects all
@@ -426,7 +512,8 @@ value of EDIT-WINDOW-NR is ignored."
 
 (defecb-window-dedicator ecb-set-analyse-buffer ecb-analyse-buffer-name
   "Display the analyse buffer in current window and make window dedicated."
-  (add-hook 'ecb-current-buffer-sync-hook-internal 'ecb-analyse-buffer-sync)
+  (ecb-activate-ecb-autocontrol-function ecb-analyse-buffer-sync-delay
+                                         'ecb-analyse-buffer-sync)
   (switch-to-buffer ecb-analyse-buffer-name))
 
 (defun ecb-maximize-window-analyse ()
@@ -522,7 +609,9 @@ completions. This means that this node should be highlighted when mouse is
 moved over it."
   (or (equal ecb-analyse-nodedata-tag-with-pos
              (nth 1 (tree-node->data node)))
-      (= (tree-node->type node) ecb-analyse-nodetype-completions)))
+      (member (tree-node->type node)
+              (list ecb-analyse-nodetype-completions
+                    ecb-analyse-nodetype-localvars))))
 
 (defun ecb-analyse-create-menu (node)
   "Return a popup-menu suitable for NODE."
@@ -530,9 +619,11 @@ moved over it."
          (tag-p (not (equal (nth 1 data) ecb-analyse-nodedata-no-tag)))
          (tag-with-pos-p (equal (nth 1 data) ecb-analyse-nodedata-tag-with-pos))
          (nodetype (nth 2 data)))
-    (delq nil (list (if (equal nodetype ecb-analyse-nodetype-completions)
-                        '(ecb-analyse-complete "Complete"))
-                    (if tag-p
+    (delq nil (list (if (member nodetype (list
+                                          ecb-analyse-nodetype-completions
+                                          ecb-analyse-nodetype-localvars))
+                        '(ecb-analyse-complete/insert "Complete/insert"))
+                    (if tag-p 
                         '(ecb-analyse-show-tag-info "Show tag info"))
                     (if tag-with-pos-p
                         '(ecb-analyse-jump-to-tag "Jump to tag"))))))
@@ -578,8 +669,6 @@ analyse-buffer."
    :frame ecb-frame
    :mouse-action-trigger ecb-tree-mouse-action-trigger
    :is-click-valid-fn 'ecb-interpret-mouse-click
-   ;; TODO: Klaus Berndl <klaus.berndl@sdm.de>: Maybe we should make own
-   ;; callbacks for analyse...
    :node-selected-fn 'ecb-tree-buffer-node-select-callback
    :node-expanded-fn 'ecb-tree-buffer-node-expand-callback
    :node-collapsed-fn 'ecb-tree-buffer-node-collapsed-callback
@@ -591,6 +680,9 @@ analyse-buffer."
    :menu-creator 'ecb-analyse-menu-creator
    :menu-titles (ecb-analyse-gen-menu-title-creator)
    :modeline-menu-creator 'ecb-common-tree-buffer-modeline-menu-creator
+   :sticky-parent-p ecb-tree-make-parent-node-sticky
+   :sticky-indent-string ecb-tree-stickynode-indent-string
+   :sticky-parent-fn nil
    :trunc-lines (ecb-member-of-symbol/value-list ecb-analyse-buffer-name
                                                  ecb-tree-truncate-lines)
    :read-only t
