@@ -5,11 +5,9 @@
 ;;                           Kevin A. Burton,
 ;;                           Free Software Foundation, Inc.
 
-;; Author: Jesper Nordenberg <mayhem@home.se>
-;;         Klaus Berndl <klaus.berndl@sdm.de>
+;; Author: Klaus Berndl <klaus.berndl@sdm.de>
 ;;         Kevin A. Burton <burton@openprivacy.org>
 ;; Maintainer: Klaus Berndl <klaus.berndl@sdm.de>
-;;             Kevin A. Burton <burton@openprivacy.org>
 ;; Keywords: browser, code, programming, tools
 ;; Created: 2001
 
@@ -26,7 +24,7 @@
 ;; GNU Emacs; see the file COPYING.  If not, write to the Free Software
 ;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-;; $Id: ecb-face.el,v 1.23 2005/02/28 11:31:58 berndl Exp $
+;; $Id: ecb-face.el,v 1.25 2009/05/03 13:16:11 berndl Exp $
 
 ;;; Commentary:
 
@@ -223,6 +221,80 @@ Changes take first effect after finishing and reactivating ECB!"
                 (face :tag "Special face"
                       :value ecb-history-general-face)))
 
+;; TODO: Klaus Berndl <klaus.berndl@sdm.de>: add the following three faces
+;; incl. their options to the texi
+(defface ecb-history-bucket-node-face (ecb-face-default nil nil nil
+                                                        'ecb-bucket-node-face)
+  "*Face used for displaying a bucket-node in the ECB-history-buffer.
+
+In GNU Emacs 21.X this face inherits from the face 'ecb-bucket-node-face'.
+
+With XEmacs and GNU Emacs 20.X there is no inheritance-feature so if the
+buckets in the ECB-history-buffer should be displayed with the same basic
+attributes set by 'ecb-bucket-node-face' this set of basic attributes have
+to be set in 'ecb-history-bucket-node-face' too!"
+  :group 'ecb-faces)
+
+(defcustom ecb-history-bucket-node-face 'ecb-history-bucket-node-face
+  "*Basic face for displaying a bucket-node in the ECB-history-buffer.
+This defines the basic face for the bucket-nodes used to bucketize the
+history-entries as defined with the option `ecb-history-make-buckets'.
+
+Changes take first effect after finishing and reactivating ECB!"
+  :group 'ecb-face-options
+  :group 'ecb-history
+  :type '(radio (const :tag "Use ecb-bucket-node-face"
+                       :value ecb-bucket-node-face)
+                (face :tag "Special face"
+                      :value ecb-history-bucket-node-face)))
+
+(defface ecb-history-indirect-buffer-face (ecb-face-default nil nil t
+                                                            'ecb-history-general-face)
+  "*Define a face for displaying indirect buffers in the history buffer.
+
+In GNU Emacs 21.X this face inherits from the face 'ecb-history-general-face'.
+
+With XEmacs and GNU Emacs 20.X there is no inheritance-feature so if the
+buckets in the ECB-history-buffer should be displayed with the same basic
+attributes set by 'ecb-history-general-face' this set of basic attributes have
+to be set in 'ecb-history-indirect-buffer-face' too!"
+  :group 'ecb-faces)
+ 
+(defcustom ecb-history-indirect-buffer-face 'ecb-history-indirect-buffer-face
+  "*Face for indirect buffers in the history buffer."
+  :group 'ecb-history
+  :group 'ecb-face-options
+  :type '(radio (const :tag "Use ecb-history-general-face"
+                       :value ecb-history-general-face)
+                (face :tag "Special face"
+                      :value ecb-history-indirect-buffer-face)))
+
+(defface ecb-history-dead-buffer-face (ecb-face-default nil nil nil
+                                                        'ecb-history-general-face
+                                                        "gray60"
+                                                        "gray60"
+                                                        nil nil
+                                                        nil "gray60")
+  "*Define a face for history entries pointing to dead buffers.
+
+In GNU Emacs 21.X this face inherits from the face 'ecb-history-general-face'.
+
+With XEmacs and GNU Emacs 20.X there is no inheritance-feature so if the
+buckets in the ECB-history-buffer should be displayed with the same basic
+attributes set by 'ecb-history-general-face' this set of basic attributes have
+to be set in 'ecb-history-dead-buffer-face' too!"
+  :group 'ecb-faces)
+ 
+(defcustom ecb-history-dead-buffer-face 'ecb-history-dead-buffer-face
+  "*Face for history entries pointing to dead buffers."
+  :group 'ecb-history
+  :group 'ecb-face-options
+  :type '(radio (const :tag "Use ecb-history-general-face"
+                       :value ecb-history-general-face)
+                (face :tag "Special face"
+                      :value ecb-history-dead-buffer-face)))
+
+
 (defface ecb-default-highlight-face (ecb-face-default nil nil nil
                                                       nil ;'ecb-default-general-face
                                                       "yellow" nil
@@ -393,7 +465,7 @@ Changes take first effect after finishing and reactivating ECB!"
                       :value ecb-analyse-general-face)))
 
 (defface ecb-analyse-bucket-element-face (ecb-face-default nil nil nil
-                                                           'ecb-default-general-face
+                                                           'ecb-analyse-general-face
                                                            "brown")
   "*Face used for displaying elements of buckets in the ECB-analyse-buffer.
 
@@ -419,7 +491,7 @@ Changes take first effect after finishing and reactivating ECB!"
                       :value ecb-analyse-bucket-element-face)))
 
 (defface ecb-analyse-bucket-node-face (ecb-face-default nil t nil
-                                                           'ecb-default-general-face)
+                                                        'ecb-bucket-node-face)
   "*Face used for displaying a bucket-node in the ECB-analyse-buffer.
 
 In GNU Emacs 21.X this face inherits from the face 'ecb-default-general-face'.
@@ -438,8 +510,8 @@ Completions etc. in the ECB-analyse-buffer.
 Changes take first effect after finishing and reactivating ECB!"
   :group 'ecb-face-options
   :group 'ecb-analyse
-  :type '(radio (const :tag "Use ecb-default-general-face"
-                       :value ecb-default-general-face)
+  :type '(radio (const :tag "Use ecb-bucket-node-face"
+                       :value ecb-bucket-node-face)
                 (face :tag "Special face"
                       :value ecb-analyse-bucket-node-face)))
 
@@ -460,7 +532,7 @@ jumping to it by clicking onto a node in the methods buffer."
   :type 'face)
 
 (defface ecb-source-in-directories-buffer-face (ecb-face-default nil nil nil
-                                                                 'ecb-default-general-face
+                                                                 'ecb-directories-general-face
                                                                  "medium blue"
                                                                  "LightBlue1"
                                                                  nil nil
@@ -468,19 +540,18 @@ jumping to it by clicking onto a node in the methods buffer."
   "*Define a face for displaying sources in the directories buffer."
   :group 'ecb-faces)
  
-(defcustom ecb-source-in-directories-buffer-face
-  'ecb-source-in-directories-buffer-face
+(defcustom ecb-source-in-directories-buffer-face 'ecb-source-in-directories-buffer-face
   "*Face for source files in the directories buffer."
   :group 'ecb-directories
   :group 'ecb-face-options
   :type 'face)
 
-(defface ecb-source-read-only-face (ecb-face-default nil nil t)
+(defface ecb-source-read-only-face (ecb-face-default nil nil t
+                                                     'ecb-default-general-face)
   "*Define a face for displaying read-only sources."
   :group 'ecb-faces)
  
-(defcustom ecb-source-read-only-face
-  'ecb-source-read-only-face
+(defcustom ecb-source-read-only-face 'ecb-source-read-only-face
   "*Face for read-only sources."
   :group 'ecb-sources
   :group 'ecb-directories
@@ -488,7 +559,7 @@ jumping to it by clicking onto a node in the methods buffer."
   :type 'face)
 
 (defface ecb-directory-not-accessible-face (ecb-face-default nil nil nil
-                                                             'ecb-default-general-face
+                                                             'ecb-directories-general-face
                                                              "gray60"
                                                              "gray60"
                                                              nil nil
@@ -496,8 +567,7 @@ jumping to it by clicking onto a node in the methods buffer."
   "*Define a face for displaying not accessible dirs in the directories buffer."
   :group 'ecb-faces)
  
-(defcustom ecb-directory-not-accessible-face
-  'ecb-directory-not-accessible-face
+(defcustom ecb-directory-not-accessible-face 'ecb-directory-not-accessible-face
   "*Face for not accessible dirs in the directories buffer."
   :group 'ecb-directories
   :group 'ecb-face-options
@@ -613,8 +683,6 @@ See `ecb-mode-line-data'. For XEmacs the face should inherit from the face
   :group 'ecb-face-options
   :type 'face)
 
-
-
 (silentcomp-provide 'ecb-face)
 
 ;;; ecb-face.el ends here
