@@ -9,7 +9,6 @@
 ;;         Klaus Berndl <klaus.berndl@sdm.de>
 ;;         Kevin A. Burton <burton@openprivacy.org>
 ;; Maintainer: Klaus Berndl <klaus.berndl@sdm.de>
-;;             Kevin A. Burton <burton@openprivacy.org>
 ;; Keywords: browser, code, programming, tools
 ;; Created: 2000
 
@@ -26,7 +25,7 @@
 ;; GNU Emacs; see the file COPYING.  If not, write to the Free Software
 ;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-;; $Id: ecb-util.el,v 1.135 2005/06/20 14:34:22 berndl Exp $
+;; $Id: ecb-util.el,v 1.155 2009/05/15 15:19:53 berndl Exp $
 
 ;;; Commentary:
 ;;
@@ -50,6 +49,7 @@
 ;;; ----- Silentcomp-Defs ----------------------------------
 
 ;; XEmacs
+(silentcomp-defun symbol-value-in-buffer)
 (silentcomp-defun button-release-event-p)
 (silentcomp-defun button-press-event-p)
 (silentcomp-defun event-key)
@@ -69,6 +69,7 @@
 (silentcomp-defun posn-point)
 (silentcomp-defun posn-window)
 (silentcomp-defun event-start)
+(silentcomp-defun set-window-vscroll)
 ;; XEmacs
 (silentcomp-defun make-dialog-box)
 (silentcomp-defun display-message)
@@ -81,6 +82,8 @@
 (silentcomp-defun display-images-p)
 (silentcomp-defvar tar-subfile-mode)
 (silentcomp-defvar archive-subfile-mode)
+(silentcomp-defun count-screen-lines)
+(silentcomp-defvar header-line-format)
 
 ;; timer stuff for Xemacs
 (silentcomp-defun delete-itimer)
@@ -94,7 +97,7 @@
 ;;; ----- Some constants -----------------------------------
 
 ;;;###autoload
-(defconst ecb-running-xemacs (string-match "XEmacs\\|Lucid" emacs-version))
+;;(defconst ecb-running-xemacs (string-match "XEmacs\\|Lucid" emacs-version))
 (defconst ecb-running-xemacs (featurep 'xemacs))
 
 (defconst ecb-running-unsupported-emacs (condition-case nil
@@ -102,6 +105,14 @@
                                           (error t))
   "True if running XEmacs or Emacs < 21.")
 
+(defconst ecb-running-version-22 (and (not ecb-running-unsupported-emacs)
+                                      (>= emacs-major-version 22))
+  "True if running \(X)Emacs >= version 22")
+
+(defconst ecb-running-version-23 (and (not ecb-running-unsupported-emacs)
+                                      (>= emacs-major-version 23))
+  "True if running \(X)Emacs >= version 23")
+
 (defconst ecb-temp-dir
   (file-name-as-directory
    (or (getenv "TMPDIR") (getenv "TMP") (getenv "TEMP")
@@ -123,18 +134,28 @@
   (and ecb-running-xemacs
        (file-exists-p (expand-file-name (concat ecb-ecb-dir "_pkg.el")))
        (file-exists-p (expand-file-name (concat ecb-ecb-dir "auto-autoloads.el")))))
-(defconst ecb-semantic-regular-xemacs-package-p
-  (and ecb-running-xemacs
-       ecb-semantic-dir
-       (file-exists-p (expand-file-name (concat ecb-semantic-dir "_pkg.el")))
-       (file-exists-p (expand-file-name (concat ecb-semantic-dir "auto-autoloads.el")))))
 
-(defconst ecb-images-can-be-used
-  (and (or (fboundp 'defimage)
-           (fboundp 'make-image-specifier))
-       (if (fboundp 'display-images-p)
-           (display-images-p)
-         window-system)))
+;; image support possible with current Emacs setup?
+;; This will first checked at activation-time of ECB because otherwise usage
+;; of emacs --deamon could fail...
+
+(defvar ecb-images-can-be-used nil
+  "INTERNAL - DO NOT USE AND CHANGE!")
+(defvar ecb-images-can-be-used-init-p nil
+  "INTERNAL - DO NOT USE AND CHANGE!")
+
+(defun ecb-images-can-be-used ()
+  "Not nil if images can be used with current Emacs setup."
+  (if ecb-images-can-be-used-init-p
+      ecb-images-can-be-used
+    (setq ecb-images-can-be-used-init-p t)
+    (setq ecb-images-can-be-used
+          (and (or (fboundp 'defimage)
+                   (fboundp 'make-image-specifier))
+               (if (fboundp 'display-images-p)
+                   (display-images-p)
+                 window-system)))))
+
 
 ;;; ----- Tracing ------------------------------------------
 
@@ -186,6 +207,13 @@ Unless optional argument INPLACE is non-nil, return a new string."
         (if (eq (aref newstr i) fromchar)
             (aset newstr i tochar)))
       newstr))
+  (defun ecb-substring-no-properties (string &optional start end)
+    (let* ((start (or start 0))
+           (end (or end (length string)))
+           (string (substring string start end)))
+      (set-text-properties start end nil string)
+      string))
+    
   (defun ecb-derived-mode-p (&rest modes)
     "Non-nil if the current major mode is derived from one of MODES.
 Uses the `derived-mode-parent' property of the symbol to trace backwards."
@@ -193,8 +221,16 @@ Uses the `derived-mode-parent' property of the symbol to trace backwards."
       (while (and (not (memq parent modes))
                   (setq parent (get parent 'derived-mode-parent))))
       parent))
+  (defsubst ecb-count-screen-lines (&optional beg end)
+    (let ((b (or beg (point-min)))
+          (e (or end (point-max))))
+      (count-lines b e)))
   (defalias 'ecb-frame-parameter 'frame-property)
   (defalias 'ecb-line-beginning-pos 'point-at-bol)
+  (defalias 'ecb-bolp 'bolp)
+  (defalias 'ecb-eolp 'eolp)
+  (defalias 'ecb-bobp 'bobp)
+  (defalias 'ecb-eobp 'eobp)
   (defalias 'ecb-line-end-pos 'point-at-eol)
   (defalias 'ecb-event-window 'event-window)
   (defalias 'ecb-event-point 'event-point)
@@ -225,10 +261,17 @@ Uses the `derived-mode-parent' property of the symbol to trace backwards."
     "Return non-nil if running non-interactively, i.e. in batch mode."
     noninteractive)
   (defalias 'ecb-subst-char-in-string 'subst-char-in-string)
+  (defalias 'ecb-substring-no-properties 'substring-no-properties)
   (defalias 'ecb-derived-mode-p 'derived-mode-p)
+  (defsubst ecb-count-screen-lines (&optional beg end)
+    (count-screen-lines beg end))
   (defalias 'ecb-frame-parameter 'frame-parameter)
   (defalias 'ecb-line-beginning-pos 'line-beginning-position)
   (defalias 'ecb-line-end-pos 'line-end-position)
+  (defalias 'ecb-bolp 'bolp)
+  (defalias 'ecb-eolp 'eolp)
+  (defalias 'ecb-bobp 'bobp)
+  (defalias 'ecb-eobp 'eobp)
   (defun ecb-event-window (event)
     (posn-window (event-start event)))
   (defun ecb-event-point (event)
@@ -303,100 +346,6 @@ Uses the `derived-mode-parent' property of the symbol to trace backwards."
     (delete-itimer timer))
   )
 
-;;; ----- advice stuff -------------------------------------
-
-;; TODO: Klaus Berndl <klaus.berndl@sdm.de>: Attention. Current mechanism of
-;; (de)activating the basic advices and the intelligent window advices of
-;; `ecb-advice-window-functions' independent from each other works only if
-;; both sets of functions are disjunct (because ad-activate always activates
-;; ALL advices of a function if they are not disabled!)
-(defconst ecb-basic-adviced-functions (if ecb-running-xemacs
-                                          '((delete-frame . around)
-                                            (compilation-set-window-height . around)
-                                            (shrink-window-if-larger-than-buffer . around)
-                                            (show-temp-buffer-in-current-frame . around)
-                                            (pop-to-buffer . around)
-                                            (current-window-configuration . after)
-                                            (set-window-configuration . after)
-                                            (scroll-other-window . around)
-                                            (custom-save-all . around)
-                                            (count-windows . around)
-                                            (scroll-all-mode . after))
-                                        '((delete-frame . around)
-                                          (compilation-set-window-height . around)
-                                          (resize-temp-buffer-window . around)
-                                          (shrink-window-if-larger-than-buffer . around)
-                                          (mouse-drag-vertical-line . around)
-                                          (mouse-drag-mode-line . around)
-                                          (pop-to-buffer . around)
-                                          (current-window-configuration . after)
-                                          (set-window-configuration . after)
-                                          (enlarge-window . around)
-                                          (shrink-window . around)
-                                          (tmm-prompt . around)
-                                          (scroll-other-window . around)
-                                          (custom-save-all . around)
-                                          (count-windows . around)
-                                          (scroll-all-mode . after)))
-  "These functions are always adviced if ECB is active. Each element of the
-list is a cons-cell where the car is the function-symbol and the cdr the
-advice-class \(before, around or after). If a function should be adviced with
-more than one class \(e.g. with a before and an after-advice) then for every
-class a cons must be added to this list.
-
-Every basic advice of ECB must be registered in this constant but can be
-implemented in another file!")
-
-(defun ecb-enable-advices (advice-list)
-  "Enable all advices of ADVICE-LIST. ADVICE-LIST must have the format of
-`ecb-basic-adviced-functions'."
-  (dolist (elem advice-list)
-    (ad-enable-advice (car elem) (cdr elem) 'ecb)
-    (ad-activate (car elem))))
-  
-(defun ecb-disable-advices (advice-list)
-  "Disable all advices of ADVICE-LIST. ADVICE-LIST must have the format of
-`ecb-basic-adviced-functions'."
-  (dolist (elem advice-list)
-    (ad-disable-advice (car elem) (cdr elem) 'ecb)
-    (ad-activate (car elem))))
-  
-
-(defmacro ecb-with-original-basic-functions (&rest body)
-  "Evaluates BODY with all adviced basic-functions of ECB deactivated \(means
-with their original definition). Restores always the previous state of the ECB
-adviced basic-functions, means after evaluating BODY it activates the advices
-of exactly the functions in `ecb-basic-adviced-functions'!"
-  `(unwind-protect
-       (progn
-         (ecb-disable-advices ecb-basic-adviced-functions)
-         ,@body)
-     (ecb-enable-advices ecb-basic-adviced-functions)))
-
-(defun ecb-enable-ecb-advice (function-symbol advice-type arg)
-  "If ARG is greater or equal zero then enable the adviced version of
-FUNCTION-SYMBOL. The advice must be of type of the ADVICE-TYPE which can be
-'around, 'before or 'after."
-  (if (< arg 0)
-      (progn
-        (ad-disable-advice function-symbol advice-type 'ecb)
-        (ad-activate function-symbol))
-    (ad-enable-advice function-symbol advice-type 'ecb)
-    (ad-activate function-symbol)))
-
-(defmacro ecb-with-ecb-advice (function-symbol advice-type &rest body)
-  "Evaluates BODY with the adviced version of FUNCTION-SYMBOL. The advice must
-be of type of the ADVICE-TYPE which can be 'around, 'before or 'after. Such an
-advice has to ensure that it behaves as its original version when called for
-another frame than the `ecb-frame'."
-  `(unwind-protect
-       (progn
-         (ecb-enable-ecb-advice ,function-symbol ,advice-type 1)
-         ,@body)
-     (ecb-enable-ecb-advice ,function-symbol ,advice-type -1)))
-
-(put 'ecb-with-ecb-advice 'lisp-indent-function 2)
-
 
 ;;; ----- Customize stuff ----------------------------------
 
@@ -410,52 +359,6 @@ started with -q) nil is returned."
     (require 'cus-edit)
     (ignore-errors (custom-file))))
 
-(defadvice custom-save-all (around ecb)
-  "Save the customized options completely in the background, i.e. the
-file-buffer where the value is saved \(see option `custom-file') is not parsed
-by semantic and also killed afterwards."
-  (if ecb-minor-mode
-      (let (;; XEmacs 21.4 does not set this so we do it here, to ensure that
-            ;; the custom-file is loadede in an emacs-lisp-mode buffer, s.b.
-            (default-major-mode 'emacs-lisp-mode)
-            (ecb-window-sync nil)
-            (kill-buffer-hook nil)
-            ;; we prevent parsing the custom-file
-            (semantic-before-toplevel-bovination-hook (lambda ()
-                                                        nil))
-            (semantic--before-fetch-tags-hook (lambda ()
-                                                nil))
-            (semantic-after-toplevel-cache-change-hook nil)
-            (semantic-after-partial-cache-change-hook nil))
-        ;; Klaus Berndl <klaus.berndl@sdm.de>: we must ensure that the
-        ;; current-buffer has a lisp major-mode when the kernel of
-        ;; `custom-save-all' is called because cause of a bug (IMHO) in the
-        ;; `custom-save-delete' of GNU Emacs (which loads the file returned by
-        ;; `custom-file' with `default-major-mode' set to nil which in turn
-        ;; causes that new buffer will get the major-mode of the
-        ;; current-buffer) the file `custom-file' will get the major-mode of
-        ;; the current-buffer. So when the current-buffer has for example
-        ;; major-mode `c++-mode' then the file `custom-file' will be loaded
-        ;; into a buffer with major-mode c++-mode. The function
-        ;; `custom-save-delete' then parses this buffer with (forward-sexp
-        ;; (buffer-size)) which of course fails because forward-sexp tries to
-        ;; parse the custom-file (which is an emacs-lisp-file) as a c++-file
-        ;; with c++-paren-syntax.
-        ;; Solution: Ensure that the buffer *scratch* is current when calling
-        ;; custom-save-all so we have surely a lispy-buffer and therefore we
-        ;; can be sure that custom-file is loaded as lispy-buffer.
-        (save-excursion
-          (set-buffer (get-buffer-create "*scratch*"))
-          ;; now we do the standard task
-          ad-do-it)
-        ;; now we have to kill the custom-file buffer otherwise semantic would
-        ;; parse the buffer of custom-file and the method-buffer would be
-        ;; updated with the contents of custom-file which is definitely not
-        ;; desired.
-        (ignore-errors
-          (kill-buffer (find-file-noselect (ecb-custom-file)))))
-    ad-do-it))
-
 (defun ecb-option-get-value (option &optional type)
   "Return the value of a customizable ECB-option OPTION with TYPE, where TYPE
 can either be 'standard-value \(the default-value of the defcustom) or
@@ -471,7 +374,8 @@ in exactly this sequence."
                         (get option 'standard-value))))))
     (cond ((not (listp val)) val)
           ((equal 'quote (car val)) (car (cdr val)))
-          (t (car val)))))
+;;          (t (car val)))))
+          (t (eval val)))))
 
 ;;; ----- Assoc helpers ------------------------------------
 
@@ -528,21 +432,24 @@ The elements of the list are not copied, just the list structure itself."
           (prog1 (nreverse res) (setcdr res list)))
       (car list))))
 
-(defun ecb-set-difference (list1 list2)
+(defun ecb-set-difference (list1 list2 &optional test-fcn)
   "Combine LIST1 and LIST2 using a set-difference operation.
 The result list contains all items that appear in LIST1 but not LIST2.
 This is a non-destructive function; it makes a copy of the data if necessary
-to avoid corrupting the original LIST1 and LIST2."
+to avoid corrupting the original LIST1 and LIST2.
+If TEST-FCN is not nil then it must be a function which is used to check if an
+item of LIST1 is an element of LIST2. If TEST-FCN is nil then `memq' is used."
   (if (or (null list1) (null list2)) list1
     (let ((res nil))
       (while list1
-        (or (if (numberp (car list1))
-                (apply 'ecb-member (car list1) list2)
+        (or (if test-fcn
+                (funcall test-fcn (car list1) list2)
               (memq (car list1) list2))
             (push (car list1) res))
         (pop list1))
       res)))
 
+
 (defun ecb-member (item list &optional test-fcn)
   "Find the first occurrence of ITEM in LIST.
 Return the sublist of LIST whose car is ITEM. Comparison is done via `equal'
@@ -700,25 +607,89 @@ returned."
                    (length list))
               list))))
 
+(defun ecb-aggregate-alist (alist same-predicate sort-predicate)
+  "Return ALIST as a sorted, aggregated alist.
+
+In the result all items with the same car element (according to
+SAME-PREDICATE) are aggregated together.  The alist is first sorted by
+SORT-PREDICATE which is called with two items of the alist and has to return
+not nil if item1 should be precede item2.
+
+Please note: SAME-PREDICATE gets the car of an item as argument, whereas
+SORT-PREDICATE gets two complete items as arguments!
+
+Example:
+\(ecb-aggregate-alist
+ '((a . a1) (a . a2) (b . b1) (c . c3) (a . a4) (a . a3) (b . b3) (b . b2))
+ (function string=)
+ (lambda (item1 item2)
+   (string< (symbol-name (car item1)) (symbol-name (car item2)))))
+results in
+\((a a1 a2 a4 a3) (b b1 b3 b2) (c c3))"
+  (when (not (null alist))
+    (let (same
+	  tmp-old-car
+	  tmp-same
+	  (first-time-p t)
+	  old-car)
+      (nconc
+       (apply #'nconc
+	      (mapcar
+	       (lambda (item)
+		 (cond
+		  (first-time-p
+		   (push (cdr item) same)
+		   (setq first-time-p nil)
+		   (setq old-car (car item))
+		   nil)
+		  ((funcall same-predicate (car item) old-car)
+		   (push (cdr item) same)
+		   nil)
+		  (t
+		   (setq tmp-same same
+			 tmp-old-car old-car)
+		   (setq same (list (cdr item))
+			 old-car (car item))
+		   (list (cons tmp-old-car (nreverse tmp-same))))))
+	       (sort alist (lambda (item1 item2)
+			     (funcall sort-predicate
+                                      item1 item2)))))
+       (list (cons old-car (nreverse same)))))))
+
+;; test
+;; (ecb-aggregate-alist
+;;  '((a . a1) (a . a2) (b . b1) (c . c3) (a . a4) (a . a3) (b . b3) (b . b2))
+;;  'string=
+;;  (lambda (item1 item2)
+;;    (if (string= (car item1) (car item2))
+;;        (string< (symbol-name (cdr item1)) (symbol-name (cdr item2)))
+;;      (string< (car item1) (car item2)))))
+
 ;; Maybe we should enhance this docstring ;-)
 (defun ecb-member-of-symbol/value-list (value list &optional elem-accessor
-                                              return-accessor)
+                                              return-accessor compare-fcn)
   "Returns not nil when VALUE is a member of that list which is build from
 LIST by using the symbol-value if a list-member is a symbol and otherwise the
 list-member itself. If a member then the matching elem of LIST is returned.
-But if ELEM-ACCESSOR is a function then it is used to get that part of a elem
+
+Per default comparison between VALUE and such a list-elem is done by `equal'
+unless third optional argument COMPARE-FCN is not nil: Then this function is
+used.
+
+If ELEM-ACCESSOR is a function then it is used to get that part of an elem
 of LIST for which the rule above should be applied. If RETURN-ACCESSOR is a
 function then it is used to get that part of that list-elem which is equal
 according to the rules above."
   (let ((elem-acc (or elem-accessor 'identity))
-        (return-acc (or return-accessor 'identity)))
+        (return-acc (or return-accessor 'identity))
+        (cmp-fcn (or compare-fcn 'equal)))
     (catch 'exit
       (dolist (elem list)
         (let ((case-fold-search t)
               (e (funcall elem-acc elem)))
-          (if (equal value (if (symbolp e)
-                               (symbol-value e)
-                             e))
+          (if (funcall cmp-fcn value (if (symbolp e)
+                                         (symbol-value e)
+                                       e))
               (throw 'exit (funcall return-acc elem)))
           nil)))))
 
@@ -1359,7 +1330,11 @@ of TEXT which are not set by FACE are preserved."
                                      (otherwise nil))))
                              ;; we must add the new-face in front of
                              ;; current-face to get the right merge!
-                             (append nf cf))
+                             (if (member face cf)
+                                 cf
+                               (append nf cf)
+                               )
+                             )
                            text)
       (alter-text-property 0 (length text) 'face
                            (lambda (current-face)
@@ -1375,7 +1350,9 @@ of TEXT which are not set by FACE are preserved."
                                       (otherwise nil))))
                                ;; we must add the new-face in front of
                                ;; current-face to get the right merge!
-                               (append nf cf)))
+                               (if (member face cf)
+                                   cf
+                                 (append nf cf))))
                            text))
     text))
 
@@ -1503,9 +1480,6 @@ then an empty string is returned because stripping makes no sense here."
 ;; ECB has thrown away all code which is not needed by ECB
 ;; The original code is written by Eric M. Ludlam <zappo@gnu.org>
 
-;; we need this here so we are independent of the semantic-package so we can
-;; download eieio and semantic even if the user has not installed any version
-;; of semantic.
 
 ;; Variables used in stages
 (defvar ecb-working-message nil
@@ -1625,6 +1599,18 @@ It returns the exit-status of the called PROGRAM."
   "Return the current line-number - the first line in a buffer has number 1."
   (+ (count-lines 1 (point)) (if (= (current-column) 0) 1 0)))
 
+(defun ecb-goto-line (line)
+  "Goto LINE, counting from line 1 at beginning of buffer.
+
+This function doesn't set the mark."
+  ;; Move to the specified line number in that buffer.
+  (save-restriction
+    (widen)
+    (goto-char 1)
+    (if (eq selective-display t)
+        (re-search-forward "[\n\C-m]" nil 'end (1- line))
+      (forward-line (1- line)))))
+
 (defmacro ecb-with-readonly-buffer (buffer &rest body)
   "Make buffer BUFFER current but do not display it. Evaluate BODY in buffer
 BUFFER \(not read-only an evaluation-time of BODY) and make afterwards BUFFER
@@ -1703,10 +1689,12 @@ or a buffer-object."
 buffer-local value in BUFFER then the global value of SYM is used."
   (if (fboundp 'buffer-local-value)
       (buffer-local-value sym buffer)
-    (or (cdr (assoc sym (buffer-local-variables buffer)))
-        (save-excursion
-          (set-buffer buffer)
-          (symbol-value sym)))))
+    (when ecb-running-xemacs
+      (symbol-value-in-buffer sym buffer))))
+;;     (or (cdr (assoc sym (buffer-local-variables buffer)))
+;;         (save-excursion
+;;           (set-buffer buffer)
+;;           (symbol-value sym)))))
 
 
 (defun ecb-file-content-as-string (file)
@@ -1730,12 +1718,26 @@ minor-mode `tar-subfile-mode' or `archive-subfile-mode'."
       (and (boundp 'archive-subfile-mode)
            archive-subfile-mode)))
 
+(defun ecb-buffer-file-name (&optional buffer no-indirect-buffers)
+  "Return filename of file represented by BUFFER.
+BUFFER can also be an indirect buffer - if its base buffer points to a file
+then this filename is returned.
+BUFFER can be a buffer-object or a buffer-name.
+If BUFFER is nil then current buffer is used.
+If NO-INDIRECT-BUFFERS is not nil then for indirect buffers always nil is
+returned."
+  (or (buffer-file-name buffer)
+      (and (not no-indirect-buffers)
+           (buffer-base-buffer buffer)
+           (buffer-file-name (buffer-base-buffer buffer)))))
+
+
 (defun ecb-buffer-or-file-readable-p (&optional filename)
   "Checks if a buffer or a file is a readable file in the sense of ECB which
 means either a real physical file or an auto-extracted file from an archive.
 See `ecb-current-buffer-archive-extract-p'. FILENAME is either a filename or
 nil whereas in the latter case the current-buffer is assumed."
-  (let* ((file (or filename (buffer-file-name (current-buffer)))))
+  (let* ((file (or filename (ecb-buffer-file-name (current-buffer)))))
     (or (and file (file-readable-p file))
         (and (not ecb-running-xemacs)
              (if filename
@@ -1760,43 +1762,57 @@ of `next-window'. If omitted, WINDOW defaults to the selected window. FRAME and
 WINDOW default to the selected ones. Optional second arg MINIBUF t means count
 the minibuffer window even if not active. If MINIBUF is neither t nor nil it
 means not to count the minibuffer even if it is active."
-  (if (not ecb-running-xemacs)
-      ;; TODO: Klaus Berndl <klaus.berndl@sdm.de>: There seems to be
-      ;; mysterious behavior when running our own window-list version with
-      ;; GNU Emacs >= 21.3 - especially when running an igrep when the
-      ;; igrep-buffer is already in another window. We can here savely use the
-      ;; function `window-list' because it returns an ordered list
-      (window-list frame minibuf window)
-    (setq window (or window (selected-window))
-          frame (or frame (selected-frame)))
-    (if (not (eq (window-frame window) frame))
-        (error "Window must be on frame."))
-    (let ((current-frame (selected-frame))
-          (current-point (point))
-          list)
-      (unwind-protect
-          (save-window-excursion
-            (select-frame frame)
-            ;; this is needed for correct start-point
-            (select-window window)
-            (walk-windows
-             (function (lambda (cur-window)
-                         (if (not (eq window cur-window))
-                             (setq list (cons cur-window list)))))
-             minibuf
-             'selected)
-            ;; This is needed to get the right canonical windows-order, i.e. the
-            ;; same order of windows than `walk-windows' walks through!
-            (setq list (nreverse list))
-            (setq list (cons window list)))
-        (select-frame current-frame)
-        ;; we must reset the point of the buffer which was current at call-time
-        ;; of this function
-        (goto-char current-point)))))
+  ;; At least under XEmacs 21.5 there's a problem with the advice on
+  ;; current-window-configuration -- that advice calls
+  ;; ecb-window-configuration-data, which in turn involves ecb-windows-list,
+  ;; which uses save-windows-excursion, which in 21.5-b28 is. . . a macro
+  ;; which uses current-window-configuration!
+  ;; To avoid this we run the body of this function with deactivated basic
+  ;; advices of ecb.
+   (if (not ecb-running-xemacs)
+       ;; Klaus Berndl <klaus.berndl@sdm.de>: There seems to be mysterious
+       ;; behavior when running our own window-list version with GNU Emacs >=
+       ;; 21.3 - especially when running an igrep when the igrep-buffer is
+       ;; already in another window. We can here savely use the function
+       ;; `window-list' because it returns an ordered list
+       (window-list frame minibuf window)
+     ;; TODO: Klaus Berndl <klaus.berndl@sdm.de>: the following is needed for
+     ;; XEmacs - but the best would be if we would not need
+     ;; implementing window-list, means the best would be if window-list
+     ;; returns an ordered list!
+     (ecb-with-original-basic-functions
+      (setq window (or window (selected-window))
+            frame (or frame (selected-frame)))
+      (if (not (eq (window-frame window) frame))
+          (error "Window must be on frame."))
+      (let ((current-frame (selected-frame))
+            (current-window (selected-window))
+            (current-point (point))
+            list)
+        (unwind-protect
+            (progn ;;save-window-excursion
+              (select-frame frame)
+              ;; this is needed for correct start-point
+              (select-window window)
+              (walk-windows
+               (function (lambda (cur-window)
+                           (if (not (eq window cur-window))
+                               (setq list (cons cur-window list)))))
+               minibuf
+               'selected)
+              ;; This is needed to get the right canonical windows-order, i.e. the
+              ;; same order of windows than `walk-windows' walks through!
+              (setq list (nreverse list))
+              (setq list (cons window list)))
+          (select-frame current-frame)
+          (select-window current-window)
+          ;; we must reset the point of the buffer which was current at call-time
+          ;; of this function
+          (goto-char current-point))))))
 
 (defun ecb-canonical-windows-list ()
   "Return a list of all current visible windows in the `ecb-frame' \(starting
-from the left-most top-most window) in the order `other-window' would walk
+from the left-most top-most window) in the order `next-window' would walk
 through these windows."
   (ecb-window-list ecb-frame 0 (frame-first-window ecb-frame)))
 
@@ -1817,6 +1833,120 @@ height is that fraction of the frame."
               (enlarge-window enlargement))))
     (error "Window is not alive!")))
 
+(defun ecb-window-safely-shrinkable-p (&optional window)
+  "Non-nil if the WINDOW can be shrunk without shrinking other windows.
+If WINDOW is nil or omitted, it defaults to the currently selected window."
+  (with-selected-window (or window (selected-window))
+    (let ((edges (ecb-window-edges)))
+      (or (= (nth 2 edges) (nth 2 (ecb-window-edges (previous-window))))
+	  (= (nth 0 edges) (nth 0 (ecb-window-edges (next-window))))))))
+
+(defun ecb-fit-window-to-buffer (&optional window max-height min-height)
+  "Make WINDOW the right height to display its contents exactly.
+If WINDOW is omitted or nil, it defaults to the selected window.
+If the optional argument MAX-HEIGHT is supplied, it is the maximum height
+  the window is allowed to be, defaulting to the frame height.
+If the optional argument MIN-HEIGHT is supplied, it is the minimum
+  height the window is allowed to be, defaulting to `window-min-height'.
+
+MAX-HEIGHT and MIN-HEIGHT can be also afraction between 0 and 1: then this is
+interpreted as that fraction of the frame-height of WINDOW \(or the selected
+window if WINDOW is nil).
+
+The heights in MAX-HEIGHT and MIN-HEIGHT include the mode-line and/or
+header-line."
+  (interactive)
+
+  (when (null window)
+    (setq window (selected-window)))
+  (when (null max-height)
+    (setq max-height (frame-height (window-frame window))))
+
+  (let* ((buf
+	  ;; Buffer that is displayed in WINDOW
+	  (window-buffer window))
+	 (window-height
+	  ;; The current height of WINDOW
+	  (ecb-window-full-height window)) ;; KB: was window-height
+         (max-height-norm (ecb-normalize-number max-height
+                                                (frame-height (window-frame window))))
+         (min-height-norm (and min-height
+                               (ecb-normalize-number min-height
+                                                     (frame-height (window-frame window)))))
+	 (desired-height
+	  ;; The height necessary to show the buffer displayed by WINDOW
+	  ;; (`count-screen-lines' always works on the current buffer).
+	  (with-current-buffer buf
+	    (+ (ecb-count-screen-lines)
+	       ;; If the buffer is empty, (count-screen-lines) is
+	       ;; zero.  But, even in that case, we need one text line
+	       ;; for cursor.
+	       (if (= (point-min) (point-max))
+		   1 0)
+	       ;; For non-minibuffers, count the mode-line, if any
+	       (if (and (not (window-minibuffer-p window))
+			mode-line-format)
+		   1 0)
+	       ;; Count the header-line, if any
+               (if ecb-running-xemacs
+                   0
+                 (if header-line-format 1 0)))))
+	 (delta
+	  ;; Calculate how much the window height has to change to show
+	  ;; desired-height lines, constrained by MIN-HEIGHT and MAX-HEIGHT.
+	  (- (max (min desired-height max-height-norm)
+		  (or min-height-norm window-min-height))
+	     window-height))
+	 ;; We do our own height checking, so avoid any restrictions due to
+	 ;; window-min-height.
+	 (window-min-height 1))
+
+    ;; Don't try to redisplay with the cursor at the end
+    ;; on its own line--that would force a scroll and spoil things.
+    (when (with-current-buffer buf
+	    (and (ecb-eobp) (ecb-bolp) (not (ecb-bobp))))
+      (set-window-point window (1- (window-point window))))
+
+    (save-selected-window
+      (select-window window)
+
+      ;; Adjust WINDOW to the nominally correct size (which may actually
+      ;; be slightly off because of variable height text, etc).
+      (unless (zerop delta)
+	(enlarge-window delta))
+
+      ;; Check if the last line is surely fully visible.  If not,
+      ;; enlarge the window.
+      (let ((end (with-current-buffer buf
+		   (save-excursion
+		     (goto-char (point-max))
+		     (when (and (ecb-bolp) (not (ecb-bobp)))
+		       ;; Don't include final newline
+		       (backward-char 1))
+		     (when truncate-lines
+		       ;; If line-wrapping is turned off, test the
+		       ;; beginning of the last line for visibility
+		       ;; instead of the end, as the end of the line
+		       ;; could be invisible by virtue of extending past
+		       ;; the edge of the window.
+		       (forward-line 0))
+		     (point)))))
+        (unless ecb-running-xemacs
+          (set-window-vscroll window 0))
+	(while (and (< desired-height max-height-norm)
+		    (= desired-height (window-height window))
+		    (not (pos-visible-in-window-p end window)))
+	  (enlarge-window 1)
+	  (setq desired-height (1+ desired-height)))))))
+
+(defun ecb-test-fit-window-to-buffer ()
+  (interactive)
+  (ecb-fit-window-to-buffer
+   (selected-window)
+   (if (functionp temp-buffer-max-height)
+       (funcall temp-buffer-max-height (current-buffer))
+     temp-buffer-max-height)))
+
 (defun ecb-scroll-window (point window-start)
   "Scrolls window of current buffer. The window will start at WINDOW-START and
 point will stay on POINT."
@@ -1838,11 +1968,13 @@ nothing happens and nil is returned."
 which can be either a buffer-object or a buffer-name. If that window is not
 visible then BODY is not evaluated and the symbol 'window-not-visible is
 returned. Otherwise the return value of BODY is returned. Runs encapsulated in
-`save-selected-window'."
+`save-selected-window' and `save-excursion'."
   `(save-selected-window
      (if (not (ecb-window-select ,buffer-or-name))
          'window-not-visible
-       ,@body)))
+       (save-excursion
+         (set-buffer ,buffer-or-name)
+         ,@body))))
 
 (put 'ecb-exec-in-window 'lisp-indent-function 1)
 
@@ -1862,25 +1994,16 @@ visible in the `ecb-frame'."
         buf-list))
 
 
-(defun ecb-window-number (&optional window)
-  "Return the number of WINDOW or - if nil - of the current selected window.
-The left-top-most window of the ecb-frame has number 0. The other windows have
-the same ordering as `other-window' would walk through the frame."
-  (1- (length (memq (or window (selected-window))
-                    (nreverse (if ecb-running-xemacs
-                                  ;; TODO: Klaus Berndl <klaus.berndl@sdm.de>:
-                                  ;; There seems to be a mysterious behavior of
-                                  ;; `ecb-window-list' when changing one buffer,
-                                  ;; immediately opening another buffer and
-                                  ;; scrolling sown to bottom of the new buffer
-                                  ;; - then if this function is added to the
-                                  ;;   modeline via (:eval...) the new buffer
-                                  ;;   scrolls autom. back to beginng of
-                                  ;;   buffer.
-                                  ;; With the original window-list all is
-                                  ;; ok...
-                                  (ecb-canonical-windows-list)
-                                (window-list)))))))
+(defun ecb-window-in-window-list-number (win-list &optional window)
+  "Return the number of WINDOW in the window-list WIN-LIST.
+The left-top-most window of the frame has number 1. The other windows have
+the same ordering as `other-window' would walk through the frame.
+
+If WINDOW is nil then the currently selected window is used."
+  (let ((win-number (ecb-position win-list (or window (selected-window)))))
+    (if win-number (1+ win-number) nil)))
+
+
 
 ;;; ----- Time  stuff -----------------------------------------
 
@@ -1960,7 +2083,7 @@ defcustom-clause and has to be <= MAX-LEVEL."
                                     (ecb-create-menu-user-ext-type (1+ curr-level)
                                                                    max-level)))))))
 
-;;; byte-compiling stuff
+;;; ----- byte-compiling stuff ----------------------------
 
 (defun ecb-is-byte-compiling ()
   "Return non-nil if eval'ed during compilation.  Don't use outside
@@ -1971,7 +2094,72 @@ defcustom-clause and has to be <= MAX-LEVEL."
 (defun ecb-load-in-progress-p ()
   load-in-progress)
 
+;;; ----- User Interrupt handling -------------------------
 
+;; KB: stolen from semantic.....
+
+(defvar ecb-current-input-throw-symbol nil
+  "The current throw symbol for `ecb-exit-on-input'.")
+
+(defmacro ecb-exit-on-input (symbol &rest forms)
+  "Using SYMBOL as an argument to `throw', execute FORMS.
+If FORMS includes a call to `ecb-thow-on-input', then if a user presses any
+key during execution, this form macro will exit with the value passed to
+`ecb-throw-on-input'. If FORMS completes, then the return value is the same as
+`progn'."
+  `(let ((ecb-current-input-throw-symbol ,symbol))
+     (catch ,symbol
+       ,@forms)))
+(put 'ecb-exit-on-input 'lisp-indent-function 1)
+
+(defmacro ecb-throw-on-input (from &optional value)
+  "Exit with `throw' when in `ecb-exit-on-input' on user input.
+FROM is an indication of where this function is called. Optional arg VALUE is
+what should be thrown out and both are are combined in a cons-cell and passed
+to `throw'. It is recommended to add as FROM the name of the function calling
+this one or a descriptive symbol which indicates part of a code has been
+interrupted..
+
+Example: \(ecb-throw-on-input 'test-inner-loop \"test\") would throw a
+cons-cell \('test-inner-loop . \"test\")"
+  `(when (and ecb-current-input-throw-symbol
+              (or (input-pending-p) (accept-process-output)))
+     (throw ecb-current-input-throw-symbol (cons ,from ,value))))
+
+
+(defun ecb-test-throw-on-input ()
+  "Test that throw on input will work."
+  (interactive)
+  (ecb-throw-on-input 'done-die)
+  (message "Exit Code: %s"
+	   (ecb-exit-on-input 'testing
+	     (let ((inhibit-quit nil)
+		   (message-log-max nil))
+	       (while t
+		 (message "Looping ...")
+                 ;; with the following line it isn't interruptable... so if
+                 ;; you call a funtion which do not return you never reach the
+                 ;; throw-part and no interruption takes place (could be if you
+                 ;; run external processes!)
+                 ;; So this mechanism is better than nothing but not really
+                 ;; good... we need the `while-no-input'-macro...
+                 ;;(while t nil)
+		 (ecb-throw-on-input 'test-inner-loop "test")
+                 )
+	       'exit))))
+
+;; (defun ecb-test-throw-on-input-new ()
+;;   "Test that while-no-input will work even better."
+;;   (interactive)
+;;   (message "Exit Code: %s"
+;; 	   (while-no-input
+;; 	     (let ((inhibit-quit nil)
+;; 		   (message-log-max nil))
+;; 	       (while t
+;; 		 (message "Looping ...")
+;;                  (while t nil)
+;; 		 "test")
+;; 	       'exit))))
 
 
 ;;; ----- Provide ------------------------------------------
